import os

import pytest
import numpy as np

from astropy.table import Table
from astropy.io import fits
from .helpers import generate_wcs
from .. import FITSFigure

HEADER = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/2d_fits', '1904-66_TAN.hdr')

tab = Table({'RA': [347., 349.], 'DEC': [-68., -68]})

GOOD_INPUT = [[1, 2],
              [[1, 2], [3, 4]],
              [np.arange(2), np.arange(2)],
              [tab['RA'], tab['DEC']]
              ]

BAD_INPUT = [[1, ['s', 'e']],
             [np.arange(2), np.sum],
             [tab['RA'], 'ewr']
             ]


@pytest.mark.parametrize(('inputval'), GOOD_INPUT)
def test_pixel_coords(inputval):
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.show_markers(inputval[0], inputval[1])
    f.close()


@pytest.mark.parametrize(('inputval'), GOOD_INPUT)
def test_wcs_coords(inputval):
    wcs = generate_wcs(HEADER)
    header = fits.Header.fromtextfile(HEADER)
    wcs.naxis1 = header['NAXIS1']
    wcs.naxis2 = header['NAXIS2']
    f = FITSFigure(wcs)
    f.show_markers(inputval[0], inputval[1])
    f.close()


@pytest.mark.parametrize(('inputval'), BAD_INPUT)
def test_pixel_coords_bad(inputval):
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    with pytest.raises(Exception) as exc:
        f.show_markers(inputval[0], inputval[1])
    assert exc.value.args[0] == "x and y must be the same size"
    f.close()


@pytest.mark.parametrize(('inputval'), BAD_INPUT)
def test_wcs_coords_bad(inputval):
    wcs = generate_wcs(HEADER)
    header = fits.Header.fromtextfile(HEADER)
    wcs.naxis1 = header['NAXIS1']
    wcs.naxis2 = header['NAXIS2']
    f = FITSFigure(wcs)
    with pytest.raises(Exception) as exc:
        f.show_markers(inputval[0], inputval[1])
    f.close()
    assert exc.value.args[0] == "x and y must be the same size"



from .decorators import auto_refresh


class Frame(object):

    @auto_refresh
    def __init__(self, parent):
        self.ax = parent.ax

    @auto_refresh
    def set_linewidth(self, linewidth):
        """
        Set line width of the frame.

        Parameters
        ----------
        linewidth:
            The linewidth to use for the frame.
        """
        self.ax.coords.frame.set_linewidth(linewidth)

    @auto_refresh
    def set_color(self, color):
        """
        Set color of the frame.

        Parameters
        ----------
        color:
            The color to use for the frame.
        """
        self.ax.coords.frame.set_color(color)



from astropy import log

PLATE_CAREE_ERROR = """
Projection is Plate Caree (-CAR) and CRVALy is not zero. This can be interpreted
either according to Wells (1981) or Calabretta (2002). The former defines the
projection as rectilinear regardless of the value of CRVALy, whereas the latter
defines the projection as rectilinear only when CRVALy is zero. You will need to
specify the convention to assume by setting either convention='wells' or
convention='calabretta' when initializing the FITSFigure instance.
""".strip()


def check(header, convention=None, dimensions=[0, 1]):

    ix = dimensions[0] + 1
    iy = dimensions[1] + 1

    # If header does not contain CTYPE keywords, assume that the WCS is
    # missing or incomplete, and replace it with a 1-to-1 pixel mapping
    if 'CTYPE%i' % ix not in header or 'CTYPE%i' % iy not in header:
        log.warning("No WCS information found in header - using pixel coordinates")
        header['CTYPE%i' % ix] = 'PIXEL'
        header['CTYPE%i' % iy] = 'PIXEL'
        header['CRVAL%i' % ix] = 0.
        header['CRVAL%i' % iy] = 0.
        header['CRPIX%i' % ix] = 0.
        header['CRPIX%i' % iy] = 0.
        header['CDELT%i' % ix] = 1.
        header['CDELT%i' % iy] = 1.

    if header['CTYPE%i' % ix][4:] == '-CAR' and header['CTYPE%i' % iy][4:] == '-CAR':

        if header['CTYPE%i' % ix][:4] == 'DEC-' or header['CTYPE%i' % ix][1:4] == 'LAT':
            ilat = ix
        elif header['CTYPE%i' % iy][:4] == 'DEC-' or header['CTYPE%i' % iy][1:4] == 'LAT':
            ilat = iy
        else:
            ilat = None

        if ilat is not None and header['CRVAL%i' % ilat] != 0:

            if convention == 'calabretta':
                pass  # we don't need to do anything
            elif convention == 'wells':
                if 'CDELT%i' % ilat not in header:
                    raise Exception("Need CDELT%i to be present for wells convention" % ilat)
                crpix = header['CRPIX%i' % ilat]
                crval = header['CRVAL%i' % ilat]
                cdelt = header['CDELT%i' % ilat]
                crpix = crpix - crval / cdelt
                header['CRPIX%i' % ilat] = crpix
                header['CRVAL%i' % ilat] = 0.
            else:
                raise Exception(PLATE_CAREE_ERROR)

    return header



import numpy as np

from astropy.convolution import (convolve as astropy_convolve,
                                 Gaussian2DKernel, Box2DKernel)


def convolve(image, smooth=3, kernel='gauss'):

    if smooth is None and isinstance(kernel, str) and kernel in ['box', 'gauss']:
        return image

    if smooth is not None and not np.isscalar(smooth):
        raise ValueError("smooth= should be an integer - for more complex "
                         "kernels, pass an array containing the kernel "
                         "to the kernel= option")

    # The Astropy convolution doesn't treat +/-Inf values correctly yet, so we
    # convert to NaN here.

    image_fixed = np.array(image, dtype=float, copy=True)
    image_fixed[np.isinf(image)] = np.nan

    if isinstance(kernel, str):
        if kernel == 'gauss':
            kernel = Gaussian2DKernel(smooth, x_size=smooth * 5, y_size=smooth * 5)
        elif kernel == 'box':
            kernel = Box2DKernel(smooth, x_size=smooth * 5, y_size=smooth * 5)
        else:
            raise ValueError("Unknown kernel: {0}".format(kernel))
    else:
        kernel = kernel

    return astropy_convolve(image, kernel, boundary='extend')



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, division, unicode_literals

from .decorators import auto_refresh, fixdocstring

__all__ = ['TickLabels']


class TickLabels(object):

    def __init__(self, parent):
        self._figure = parent._figure
        self._ax = parent.ax
        self._wcs = parent.ax.wcs
        self.x = parent.x
        self.y = parent.y
        self.set_style('plain')

    @auto_refresh
    def set_xformat(self, xformat):
        """
        Set the format of the x-axis tick labels.

        If the x-axis type is ``longitude`` or ``latitude``, then the options
        are:

            * ``d.ddddd`` - decimal degrees, where the number of decimal places can be varied
            * ``hh`` or ``dd`` - hours (or degrees)
            * ``hh:mm`` or ``dd:mm`` - hours and minutes (or degrees and arcminutes)
            * ``hh:mm:ss`` or ``dd:mm:ss`` - hours, minutes, and seconds (or degrees, arcminutes, and arcseconds)
            * ``hh:mm:ss.ss`` or ``dd:mm:ss.ss`` - hours, minutes, and seconds (or degrees, arcminutes, and arcseconds), where the number of decimal places can be varied.

        If the x-axis type is ``scalar``, then the format should be a valid
        python string format beginning with a ``%``.

        If one of these arguments is not specified, the format for that axis
        is left unchanged.
        """
        if 'dd.' in xformat:
            xformat = xformat.replace('ddd.', 'd.').replace('dd.', 'd.')
        self._ax.coords[self.x].set_major_formatter(xformat)

    @auto_refresh
    def set_yformat(self, yformat):
        """
        Set the format of the y-axis tick labels.

        If the y-axis type is ``longitude`` or ``latitude``, then the options
        are:

            * ``d.ddddd`` - decimal degrees, where the number of decimal places can be varied
            * ``hh`` or ``dd`` - hours (or degrees)
            * ``hh:mm`` or ``dd:mm`` - hours and minutes (or degrees and arcminutes)
            * ``hh:mm:ss`` or ``dd:mm:ss`` - hours, minutes, and seconds (or degrees, arcminutes, and arcseconds)
            * ``hh:mm:ss.ss`` or ``dd:mm:ss.ss`` - hours, minutes, and seconds (or degrees, arcminutes, and arcseconds), where the number of decimal places can be varied.

        If the y-axis type is ``scalar``, then the format should be a valid
        python string format beginning with a ``%``.

        If one of these arguments is not specified, the format for that axis
        is left unchanged.
        """
        if 'dd.' in yformat:
            yformat = yformat.replace('ddd.', 'd.').replace('dd.', 'd.')
        self._ax.coords[self.y].set_major_formatter(yformat)

    @auto_refresh
    def set_style(self, style):
        """
        Set the format of the x-axis tick labels.

        This can be 'colons' or 'plain':

            * 'colons' uses colons as separators, for example 31:41:59.26 +27:18:28.1
            * 'plain' uses letters and symbols as separators, for example 31h41m59.26s +27º18'28.1"
        """

        if style not in ['colons', 'plain']:
            raise Exception("Label style should be one of colons/plain")

        self.style = style
        for coord in [self.x, self.y]:
            coord_type = self._ax.coords[coord].coord_type
            if coord_type in ['longitude', 'latitude']:
                if style == 'colons':
                    sep = (':', ':', '')
                else:
                    sep = None

                self._ax.coords[coord].set_separator(sep)

    @auto_refresh
    @fixdocstring
    def set_font(self, **kwargs):
        """
        Set the font of the tick labels.

        Parameters
        ----------

        common: family, style, variant, stretch, weight, size, fontproperties

        Notes
        -----

        Default values are set by matplotlib or previously set values if
        set_font has already been called. Global default values can be set by
        editing the matplotlibrc file.
        """
        self._ax.coords[self.x].set_ticklabel(**kwargs)
        self._ax.coords[self.y].set_ticklabel(**kwargs)

    @auto_refresh
    def show(self):
        """
        Show the x- and y-axis tick labels.
        """
        self.show_x()
        self.show_y()

    @auto_refresh
    def hide(self):
        """
        Hide the x- and y-axis tick labels.
        """
        self.hide_x()
        self.hide_y()

    @auto_refresh
    def show_x(self):
        """
        Show the x-axis tick labels.
        """
        self._ax.coords[self.x].set_ticklabel_visible(True)

    @auto_refresh
    def hide_x(self):
        """
        Hide the x-axis tick labels.
        """
        self._ax.coords[self.x].set_ticklabel_visible(False)

    @auto_refresh
    def show_y(self):
        """
        Show the y-axis tick labels.
        """
        self._ax.coords[self.y].set_ticklabel_visible(True)

    @auto_refresh
    def hide_y(self):
        """
        Hide the y-axis tick labels.
        """
        self._ax.coords[self.y].set_ticklabel_visible(False)

    @auto_refresh
    def set_xposition(self, position):
        """
        Set the position of the x-axis tick labels ('top' or 'bottom')
        """
        if position == 'bottom':
            self._ax.coords[self.x].set_ticklabel_position('b')
        elif position == 'top':
            self._ax.coords[self.x].set_ticklabel_position('t')
        else:
            raise ValueError("position should be one of 'top' or 'bottom'")

    @auto_refresh
    def set_yposition(self, position):
        """
        Set the position of the y-axis tick labels ('left' or 'right')
        """
        if position == 'left':
            self._ax.coords[self.y].set_ticklabel_position('l')
        elif position == 'right':
            self._ax.coords[self.y].set_ticklabel_position('r')
        else:
            raise ValueError("position should be one of 'left' or 'right'")



import numpy as np

from ..core import FITSFigure


def test_nan_color_copy():
    """
    Regression test to ensure that NaN values set in one image don't affect
    global Matplotlib colormap.
    """

    data = np.zeros((16, 16))

    f1 = FITSFigure(data)
    f1.show_grayscale()
    f1.set_nan_color('blue')

    f2 = FITSFigure(data)
    f2.show_grayscale()
    f2.set_nan_color('red')

    assert f1.image.get_cmap()._rgba_bad == (0.0, 0.0, 1.0, 1.0)
    assert f2.image.get_cmap()._rgba_bad == (1.0, 0.0, 0.0, 1.0)


def test_stretches():

    # Regression test to make sure none of the stretches crash

    data = np.arange(256).reshape((16, 16))
    f = FITSFigure(data)
    f.show_grayscale()
    f.show_grayscale(stretch='linear')
    f.show_grayscale(stretch='sqrt')
    f.show_grayscale(stretch='log')
    f.show_grayscale(stretch='arcsinh')
    f.show_grayscale(stretch='power')
    f.show_grayscale(stretch='log', vmid=-10)
    f.show_grayscale(stretch='arcsinh', vmid=10)
    f.show_grayscale(stretch='power', exponent=3.0)
    f.close()



import os

import pytest
import numpy as np
from astropy import units as u
from astropy.io import fits

from .. import FITSFigure


ROOT = os.path.dirname(os.path.abspath(__file__))
HEADER_DIR = os.path.join(ROOT, 'data/2d_fits')
HEADER = fits.Header.fromtextfile(os.path.join(HEADER_DIR, '1904-66_TAN.hdr'))
HDU = fits.PrimaryHDU(np.zeros((16, 16)), HEADER)


def test_scalebar_add_invalid():
    f = FITSFigure(HDU)
    with pytest.raises(TypeError):
        f.add_scalebar()


def test_scalebar_addremove():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.remove_scalebar()
    f.add_scalebar(0.1)
    f.close()


def test_scalebar_showhide():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.hide()
    f.scalebar.show(0.1)
    f.close()


def test_scalebar_length():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_length(0.01)
    f.scalebar.set_length(0.1)
    f.close()


@pytest.mark.parametrize('quantity', [1 * u.arcsec, u.arcsec, 2 * u.degree, 5 * u.radian])
def test_scalebar_length_quantity(quantity):
    f = FITSFigure(HDU)
    f.add_scalebar(quantity)
    f.scalebar.set_length(quantity)
    f.close()


def test_scalebar_label():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_label('1 pc')
    f.scalebar.set_label('5 AU')
    f.scalebar.set_label('2"')
    f.close()


def test_scalebar_corner():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    for corner in ['top', 'bottom', 'left', 'right', 'top left', 'top right',
                   'bottom left', 'bottom right']:
        f.scalebar.set_corner(corner)
    f.close()


def test_scalebar_frame():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_frame(True)
    f.scalebar.set_frame(False)
    f.close()


def test_scalebar_color():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_color('black')
    f.scalebar.set_color('#003344')
    f.scalebar.set_color((1.0, 0.4, 0.3))
    f.close()


def test_scalebar_alpha():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_alpha(0.1)
    f.scalebar.set_alpha(0.2)
    f.scalebar.set_alpha(0.5)
    f.close()


def test_scalebar_font():
    f = FITSFigure(HDU)
    f.add_scalebar(0.1)
    f.scalebar.set_font(size='small', weight='bold', stretch='normal',
                        family='serif', style='normal', variant='normal')
    f.close()


def test_regression_exception_type():
    # In Matplotlib 1.5, the exception type changed for when a property doesn't
    # exist, so we need to catch both AttributeError and TypeError.
    f = FITSFigure(HDU)
    with pytest.warns(UserWarning, match='Scalebar does not have attribute family'):
        f.add_scalebar(0.1, family='serif')
    f.close()



import pytest
import numpy as np

from .. import FITSFigure

# Test simple contour generation with Numpy example


@pytest.mark.parametrize(('filled'), [True, False])
def test_numpy_contour(filled):
    data = np.arange(256).reshape((16, 16))
    f = FITSFigure(data)
    f.show_grayscale()
    f.show_contour(data, levels=np.linspace(1., 254., 10), filled=filled)
    f.close()



import os

import pytest
import numpy as np
from .helpers import generate_hdu

from .. import FITSFigure

ROOT = os.path.dirname(os.path.abspath(__file__))
HEADER_DIR = os.path.join(ROOT, 'data/2d_fits')
REFERENCE = os.path.join(HEADER_DIR, '1904-66_TAN.hdr')


def test_tick_labels_show_hide():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.tick_labels.hide()
    f.tick_labels.show()
    f.tick_labels.hide_x()
    f.tick_labels.show_x()
    f.tick_labels.hide_y()
    f.tick_labels.show_y()
    f.close()


def test_tick_labels_format_scalar():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.tick_labels.set_xformat('%i')
    f.tick_labels.set_yformat('%i')
    f.close()


def test_tick_labels_position():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.tick_labels.set_xposition('top')
    f.tick_labels.set_xposition('bottom')
    f.tick_labels.set_yposition('right')
    f.tick_labels.set_yposition('left')
    f.close()


def test_tick_labels_position_invalid():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    with pytest.raises(ValueError):
        f.tick_labels.set_xposition('right')
    with pytest.raises(ValueError):
        f.tick_labels.set_xposition('left')
    with pytest.raises(ValueError):
        f.tick_labels.set_yposition('top')
    with pytest.raises(ValueError):
        f.tick_labels.set_yposition('bottom')
    f.close()


def test_tick_labels_font():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.tick_labels.set_font(size='small', weight='bold', stretch='normal',
                           family='serif', style='normal', variant='normal')
    f.close()


def test_single_d_format():
    hdu = generate_hdu(REFERENCE)
    f = FITSFigure(hdu)
    f.show_grayscale(vmin=0, vmax=1)
    f.tick_labels.set_yformat('d.d')
    f.save('test_label_format.png')



import os

import pytest
import numpy as np
from astropy import units as u
from astropy.io import fits

from .. import FITSFigure
from ..overlays import Beam


header_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/2d_fits')

HEADER = fits.Header.fromtextfile(os.path.join(header_dir, '1904-66_TAN.hdr'))
HDU = fits.PrimaryHDU(np.zeros((16, 16)), HEADER)


def test_beam_add_remove():
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.remove_beam()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.remove_beam()
    f.close()


def test_beam_show_hide():
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.hide()
    f.beam.show(major=0.1, minor=0.04, angle=10.)
    f.close()


def test_beam_major():
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_major(0.5)
    f.beam.set_major(1.0)
    f.close()


@pytest.mark.parametrize('quantity', [u.arcsec, 5 * u.arcsec, 1 * u.degree, 1 * u.radian])
def test_beam_major_quantity(quantity):
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=quantity, minor=0.04, angle=10.)
    f.beam.set_major(quantity)
    f.close()


def test_beam_minor():
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_minor(0.05)
    f.beam.set_minor(0.08)
    f.close()


@pytest.mark.parametrize('quantity', [u.arcsec, 5 * u.arcsec, 1 * u.degree, 1 * u.radian])
def test_beam_minor_quantity(quantity):
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=quantity, angle=10.)
    assert type(f.beam) is not list
    f.beam.set_minor(quantity)
    f.close()


def test_beam_angle():
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_angle(0.)
    f.beam.set_angle(55.)
    f.close()


@pytest.mark.parametrize('quantity', [u.arcsec, 5 * u.arcsec, 1 * u.degree, 1 * u.radian])
def test_beam_angle_quantity(quantity):
    f = FITSFigure(HDU)
    f.show_grayscale()
    f.add_beam(major=0.1, minor=0.04, angle=quantity)
    f.beam.set_angle(quantity)
    f.close()


def test_beam_corner():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    for corner in ['top', 'bottom', 'left', 'right', 'top left', 'top right',
                   'bottom left', 'bottom right']:
        f.beam.set_corner(corner)
    f.close()


def test_beam_frame():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_frame(True)
    f.beam.set_frame(False)
    f.close()


def test_beam_borderpad():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_borderpad(0.1)
    f.beam.set_borderpad(0.3)
    f.close()


def test_beam_pad():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_pad(0.1)
    f.beam.set_pad(0.3)
    f.close()


def test_beam_alpha():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_alpha(0.1)
    f.beam.set_alpha(0.2)
    f.beam.set_alpha(0.5)
    f.close()


def test_beam_color():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_color('black')
    f.beam.set_color('#003344')
    f.beam.set_color((1.0, 0.4, 0.3))
    f.close()


def test_beam_facecolor():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_facecolor('black')
    f.beam.set_facecolor('#003344')
    f.beam.set_facecolor((1.0, 0.4, 0.3))
    f.close()


def test_beam_edgecolor():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_edgecolor('black')
    f.beam.set_edgecolor('#003344')
    f.beam.set_edgecolor((1.0, 0.4, 0.3))
    f.close()


def test_beam_linestyle():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_linestyle('solid')
    f.beam.set_linestyle('dotted')
    f.beam.set_linestyle('dashed')
    f.close()


def test_beam_linewidth():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.beam.set_linewidth(0)
    f.beam.set_linewidth(1)
    f.beam.set_linewidth(5)
    f.close()


def test_beam_hatch():
    f = FITSFigure(HDU)
    f.add_beam(major=0.1, minor=0.04, angle=10.)
    for hatch in ['/', '\\', '|', '-', '+', 'x', 'o', 'O', '.', '*']:
        f.beam.set_hatch(hatch)
    f.close()


def test_beam_multiple():

    f = FITSFigure(HDU)
    f.show_grayscale()

    f.add_beam(major=0.1, minor=0.04, angle=10.)
    f.add_beam(major=0.2, minor=0.08, angle=10.)
    f.add_beam(major=0.15, minor=0.08, angle=10.)

    assert len(f.beam) == 3

    with pytest.raises(Exception) as exc:
        f.remove_beam()
    assert exc.value.args[0] == ('More than one beam present - use beam_index= '
                                 'to specify which one to remove')
    f.remove_beam(beam_index=2)

    assert len(f.beam) == 2

    f.remove_beam(beam_index=1)

    assert isinstance(f.beam, Beam)

    f.remove_beam()

    assert not hasattr(f, 'beam')

    f.close()



# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
APLpy : Astronomical Plotting Library in Python
"""

from .core import FITSFigure  # noqa
from .rgb import make_rgb_image, make_rgb_cube  # noqa

from .frame import Frame  # noqa
from .overlays import Scalebar, Beam  # noqa
from .colorbar import Colorbar  # noqa
from .grid import Grid  # noqa
from .ticks import Ticks  # noqa
from .tick_labels import TickLabels  # noqa
from .axis_labels import AxisLabels  # noqa
from .version import __version__  # noqa



import numpy as np
from astropy.tests.helper import pytest

from .. import FITSFigure

ARRAY = np.arange(256).reshape((16, 16))


def test_colorbar_invalid():
    f = FITSFigure(ARRAY)
    with pytest.raises(Exception):
        f.add_colorbar()  # no grayscale/colorscale was shown


def test_colorbar_addremove():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.remove_colorbar()
    f.add_colorbar()
    f.close()


def test_colorbar_showhide():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.hide()
    f.colorbar.show()
    f.close()


def test_colorbar_location():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.set_location('top')
    with pytest.warns(UserWarning, match='Bottom colorbar not fully implemented'):
        f.colorbar.set_location('bottom')
    with pytest.warns(UserWarning, match='Left colorbar not fully implemented'):
        f.colorbar.set_location('left')
    f.colorbar.set_location('right')
    f.close()


def test_colorbar_width():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.set_width(0.1)
    f.colorbar.set_width(0.2)
    f.colorbar.set_width(0.5)
    f.close()


def test_colorbar_pad():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.set_pad(0.1)
    f.colorbar.set_pad(0.2)
    f.colorbar.set_pad(0.5)
    f.close()


def test_colorbar_font():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.set_font(size='small', weight='bold', stretch='normal',
                        family='serif', style='normal', variant='normal')
    f.close()


def test_colorbar_axis_label():
    f = FITSFigure(ARRAY)
    f.show_grayscale()
    f.add_colorbar()
    f.colorbar.set_axis_label_text('Surface Brightness (MJy/sr)')
    f.colorbar.set_axis_label_rotation(45.)
    f.colorbar.set_axis_label_font(size='small', weight='bold', stretch='normal',
                                   family='serif', style='normal', variant='normal')
    f.colorbar.set_axis_label_pad(5.)
    f.close()



import pytest
import numpy as np

from .. import FITSFigure


def test_grid_addremove():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.remove_grid()
    f.add_grid()
    f.close()


def test_grid_showhide():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.hide()
    f.grid.show()
    f.close()


def test_grid_spacing():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.set_xspacing(1.)
    f.grid.set_xspacing('tick')
    with pytest.raises(ValueError):
        f.grid.set_xspacing('auto')
    f.grid.set_yspacing(2.)
    f.grid.set_yspacing('tick')
    with pytest.raises(ValueError):
        f.grid.set_yspacing('auto')
    f.close()


def test_grid_color():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.set_color('black')
    f.grid.set_color('#003344')
    f.grid.set_color((1.0, 0.4, 0.3))
    f.close()


def test_grid_alpha():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.set_alpha(0.0)
    f.grid.set_alpha(0.3)
    f.grid.set_alpha(1.0)
    f.close()


def test_grid_linestyle():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.set_linestyle('solid')
    f.grid.set_linestyle('dashed')
    f.grid.set_linestyle('dotted')
    f.close()


def test_grid_linewidth():
    data = np.zeros((16, 16))
    f = FITSFigure(data)
    f.add_grid()
    f.grid.set_linewidth(0)
    f.grid.set_linewidth(2)
    f.grid.set_linewidth(5)
    f.close()



import os

import pytest
import numpy as np
from astropy.io import fits

from .helpers import generate_file, generate_hdu, generate_wcs
from .. import FITSFigure

# The tests in this file check that the initialization and basic plotting do
# not crash for FITS files with 3+ dimensions. No reference images are
# required here.

ROOT = os.path.dirname(os.path.abspath(__file__))
HEADER_DIR = os.path.join(ROOT, 'data/3d_fits')
HEADERS = [os.path.join(HEADER_DIR, 'cube.hdr')]
REFERENCE = os.path.join(HEADER_DIR, 'cube.hdr')

VALID_DIMENSIONS = [(0, 1), (1, 0), (0, 2), (2, 0), (1, 2), (2, 1)]
INVALID_DIMENSIONS = [None, (1,), (0, 3), (-4, 2), (1, 1), (2, 2), (3, 3),
                      (1, 2, 3), (3, 5, 3, 2)]


# Test initialization through a filename
def test_file_init(tmpdir):
    filename = generate_file(REFERENCE, str(tmpdir))
    f = FITSFigure(filename, slices=[5])
    f.show_grayscale()
    f.close()


# Test initialization through an HDU object
def test_hdu_init():
    hdu = generate_hdu(REFERENCE)
    f = FITSFigure(hdu, slices=[5])
    f.show_grayscale()
    f.close()


# Test initialization through a WCS object
def test_wcs_init():
    wcs = generate_wcs(REFERENCE)
    with pytest.raises(ValueError) as exc:
        FITSFigure(wcs, slices=[5])
    assert exc.value.args[0] == "FITSFigure initialization via WCS objects can only be done with 2-dimensional WCS objects"


# Test initialization through an HDU object (no WCS)
def test_hdu_nowcs_init():
    data = np.zeros((16, 16, 16))
    hdu = fits.PrimaryHDU(data)
    f = FITSFigure(hdu, slices=[5])
    f.show_grayscale()
    f.close()


# Test initalization through a Numpy array (no WCS)
def test_numpy_nowcs_init():
    data = np.zeros((16, 16, 16))
    f = FITSFigure(data, slices=[5])
    f.show_grayscale()
    f.close()


# Test that initialization without specifying slices raises an exception for a
# true 3D cube
def test_hdu_noslices():
    hdu = generate_hdu(REFERENCE)
    with pytest.raises(Exception):
        FITSFigure(hdu)


# Test that initialization without specifying slices does *not* raise an
# exception if the remaining dimensions have size 1.
def test_hdu_noslices_2d():
    data = np.zeros((1, 16, 16))
    f = FITSFigure(data)
    f.show_grayscale()
    f.close()


# Now check initialization with valid and invalid dimensions. We just need to
# tes with HDU objects since we already tested that reading from files is ok.


# Test initialization with valid dimensions
@pytest.mark.parametrize(('dimensions'), VALID_DIMENSIONS)
def test_init_dimensions_valid(dimensions):
    hdu = generate_hdu(REFERENCE)
    f = FITSFigure(hdu, dimensions=dimensions, slices=[5])
    f.show_grayscale()
    f.close()


# Test initialization with invalid dimensions
@pytest.mark.parametrize(('dimensions'), INVALID_DIMENSIONS)
def test_init_dimensions_invalid(dimensions):
    hdu = generate_hdu(REFERENCE)
    with pytest.raises(ValueError):
        FITSFigure(hdu, dimensions=dimensions, slices=[5])


# Now check initialization of different WCS projections, and we check only
# valid dimensions

valid_parameters = []
for h in HEADERS:
    for d in VALID_DIMENSIONS:
        valid_parameters.append((h, d))


@pytest.mark.parametrize(('header', 'dimensions'), valid_parameters)
def test_init_extensive_wcs(tmpdir, header, dimensions):
    filename = generate_file(header, str(tmpdir))
    f = FITSFigure(filename, dimensions=dimensions, slices=[5])
    f.show_grayscale()
    f.close()


# Test that recenter works for cube slices
# TODO: remove xfail once pix2world transformations work for multidimensional
# datasets
@pytest.mark.xfail
def test_recenter_cube_slices():
    data = np.zeros((16, 16, 16))
    hdu = fits.PrimaryHDU(data)
    f = FITSFigure(hdu, slices=[5])
    f.show_grayscale()
    f.recenter(5., 5., width=3., height=3.)
    f.close()



import numpy as np
from astropy.io import fits

from .. import FITSFigure

ARRAY = np.arange(256).reshape((16, 16))


def test_convolve_default():
    hdu = fits.PrimaryHDU(ARRAY)
    f = FITSFigure(hdu)
    f.show_grayscale(smooth=3)
    f.close()


def test_convolve_gauss():
    hdu = fits.PrimaryHDU(ARRAY)
    f = FITSFigure(hdu)
    f.show_grayscale(kernel='gauss', smooth=3)
    f.close()


def test_convolve_box():
    hdu = fits.PrimaryHDU(ARRAY)
    f = FITSFigure(hdu)
    f.show_grayscale(kernel='box', smooth=3)
    f.close()


def test_convolve_custom():
    hdu = fits.PrimaryHDU(ARRAY)
    f = FITSFigure(hdu)
    f.show_grayscale(kernel=np.ones((3, 3)))
    f.close()


def test_convolve_int():
    # Regression test for aplpy/aplpy#165
    hdu = fits.PrimaryHDU(ARRAY)
    f = FITSFigure(hdu)
    f.show_grayscale(smooth=3)
    f.close()



import os
import warnings

import pytest
import numpy as np
from astropy.io import fits
from astropy.coordinates import Galactic

from .. import FITSFigure
from ..rgb import make_rgb_image, make_rgb_cube

from .test_images import BaseImageTests
from .helpers import generate_header
from .figures import figure_test

ROOT = os.path.dirname(os.path.abspath(__file__))
HEADER = os.path.join(ROOT, 'data/2d_fits', '1904-66_TAN.hdr')


class TestRGB(BaseImageTests):

    @pytest.mark.parametrize('embed_avm_tags', (False, True))
    def test_rgb(self, tmpdir, embed_avm_tags):

        # Regression test to check that RGB recenter works properly

        r_file = tmpdir.join('r.fits').strpath
        g_file = tmpdir.join('g.fits').strpath
        b_file = tmpdir.join('b.fits').strpath
        rgb_file = tmpdir.join('rgb.png').strpath

        np.random.seed(12345)

        header = fits.Header.fromtextfile(HEADER)

        r = fits.PrimaryHDU(np.random.random((12, 12)), header)
        r.writeto(r_file)

        g = fits.PrimaryHDU(np.random.random((12, 12)), header)
        g.writeto(g_file)

        b = fits.PrimaryHDU(np.random.random((12, 12)), header)
        b.writeto(b_file)

        make_rgb_image([r_file, g_file, b_file], rgb_file, embed_avm_tags=embed_avm_tags)

        if embed_avm_tags:
            f = FITSFigure(rgb_file, figsize=(7, 5))
        else:
            f = FITSFigure(r_file, figsize=(7, 5))

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            f.show_rgb(rgb_file)

        f.tick_labels.set_xformat('dd.d')
        f.tick_labels.set_yformat('dd.d')

        f.recenter(359.3, -72.1, radius=0.05)

        return f

    @pytest.mark.parametrize('north', ['default', 'galactic', 'false'])
    @figure_test
    def test_make_rgb_cube(self, tmpdir, north):

        # Regression test to check that RGB recenter works properly

        header = generate_header(os.path.join(ROOT, 'data', '2d_fits', '2MASS_k_rot.hdr'))

        header['CRPIX1'] = 6.5
        header['CRPIX2'] = 6.5

        header_r = header.copy()
        header_r['CROTA2'] = 5
        header_g = header.copy()
        header_g['CROTA2'] = 35
        header_b = header.copy()
        header_b['CROTA2'] = 70

        r_file = tmpdir.join('r.fits').strpath
        g_file = tmpdir.join('g.fits').strpath
        b_file = tmpdir.join('b.fits').strpath
        rgb_cube = tmpdir.join('rgb.fits').strpath
        rgb_file = tmpdir.join('rgb.png').strpath

        header = fits.Header.fromtextfile(HEADER)

        r = fits.PrimaryHDU(np.ones((128, 128)), header_r)
        r.writeto(r_file)

        g = fits.PrimaryHDU(np.ones((128, 128)), header_g)
        g.writeto(g_file)

        b = fits.PrimaryHDU(np.ones((128, 128)), header_b)
        b.writeto(b_file)

        if north == 'default':
            kwargs = {}
        elif north == 'galactic':
            kwargs = {'north': Galactic()}
        elif north == 'false':
            kwargs = {'north': False}

        make_rgb_cube([r_file, g_file, b_file], rgb_cube, **kwargs)

        make_rgb_image(rgb_cube, rgb_file, embed_avm_tags=True,
                       vmin_r=0, vmax_r=1, vmin_g=0, vmax_g=1, vmin_b=0, vmax_b=1)

        f = FITSFigure(rgb_file, figsize=(3, 3))

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            f.show_rgb(rgb_file)

        f.tick_labels.hide()
        f.axis_labels.hide()
        f.add_grid()

        return f



from astropy import log
from astropy import wcs

from .decorators import auto_refresh


class Regions(object):
    """
    Regions sub-class of APLpy.

    Used for overplotting various shapes and annotations on APLpy
    fitsfigures.

    Example:
    # DS9 region file called "test.reg"
    # (the coordinates are around l=28 in the Galactic Plane)
    # Filename: test.fits
    fk5
    box(18:42:48.262,-04:01:17.91,505.668",459.714",0) # color=red dash=1
    point(18:42:51.797,-03:59:44.82) # point=x color=red dash=1
    point(18:42:50.491,-04:03:09.39) # point=box color=red dash=1
    # vector(18:42:37.433,-04:02:10.77,107.966",115.201) vector=1 color=red dash=1
    ellipse(18:42:37.279,-04:02:11.92,26.4336",40.225",0) # color=red dash=1
    polygon(18:42:59.016,-03:58:22.06,18:42:58.219,-03:58:11.30,18:42:57.403,-03:58:35.86,18:42:58.094,-03:58:57.69,18:42:59.861,-03:58:41.60,18:42:59.707,-03:58:23.21) # color=red dash=1
    point(18:42:52.284,-04:00:02.80) # point=diamond color=red dash=1
    point(18:42:46.561,-03:58:01.57) # point=circle color=red dash=1
    point(18:42:42.615,-03:58:25.84) # point=cross color=red dash=1
    point(18:42:42.946,-04:01:44.74) # point=arrow color=red dash=1
    point(18:42:41.961,-03:57:26.16) # point=boxcircle color=red dash=1
    # text(18:42:41.961,-03:57:26.16) text={This is text} color=red

    Code:
    import aplpy
    import regions

    ff = aplpy.FITSFigure("test.fits")
    ff.show_grayscale()
    ff.show_regions('test.reg')

    """

    @auto_refresh
    def show_regions(self, region_file, layer=False, **kwargs):
        """
        Overplot regions as specified in the region file.

        Parameters
        ----------

        region_file: string or pyregion.ShapeList
            Path to a ds9 regions file or a ShapeList already read
            in by pyregion.

        layer: str, optional
            The name of the layer

        kwargs
            Additional keyword arguments, e.g. zorder, will be passed to the
            ds9 call and onto the patchcollections.
        """

        PC, TC = ds9(region_file, flatten_header(self._header), **kwargs)

        PC.add_to_axes(self.ax)
        TC.add_to_axes(self.ax)

        if layer:
            region_set_name = layer
        else:
            self._region_counter += 1
            region_set_name = 'region_set_' + str(self._region_counter)

        self._layers[region_set_name] = PC
        self._layers[region_set_name + "_txt"] = TC


def ds9(region_file, header, zorder=3, **kwargs):
    """
    Wrapper to return a PatchCollection given a ds9 region file
    and a fits header.

    zorder - defaults to 3 so that regions are on top of contours
    """

    try:
        import pyregion
    except Exception:
        raise ImportError("The pyregion package is required to load region files")

    # read region file
    if isinstance(region_file, str):
        rr = pyregion.open(region_file)
    elif isinstance(region_file, pyregion.ShapeList):
        rr = region_file
    else:
        raise Exception("Invalid type for region_file: %s - should be string or pyregion.ShapeList" % type(region_file))

    if isinstance(header, wcs.WCS):
        header = header.to_header()

    # convert coordinates to image coordinates
    rrim = rr.as_imagecoord(header)

    # pyregion and aplpy both correct for the FITS standard origin=1,1
    # need to avoid double-correcting. Also, only some items in `coord_list`
    # are pixel coordinates, so which ones should be corrected depends on the
    # shape.
    for r in rrim:
        if r.name == 'polygon':
            correct = range(len(r.coord_list))
        elif r.name == 'line':
            correct = range(4)
        elif r.name in ['rotbox', 'box', 'ellipse', 'annulus', 'circle', 'panda', 'pie', 'epanda', 'text', 'point', 'vector']:
            correct = range(2)
        else:
            log.warning("Unknown region type '{0}' - please report to the developers")
            correct = range(2)
        for i in correct:
            r.coord_list[i] += 1

    if 'text_offset' in kwargs:
        text_offset = kwargs['text_offset']
        del kwargs['text_offset']
    else:
        text_offset = 5.0

    # grab the shapes to overplot
    pp, aa = rrim.get_mpl_patches_texts(text_offset=text_offset)

    PC = ArtistCollection(pp, **kwargs)  # preserves line style (dashed)
    TC = ArtistCollection(aa, **kwargs)
    PC.set_zorder(zorder)
    TC.set_zorder(zorder)

    return PC, TC


class ArtistCollection():
    """
    Matplotlib collections can't handle Text.
    This is a barebones collection for text objects
    that supports removing and making (in)visible
    """

    def __init__(self, artistlist):
        """
        Pass in a list of matplotlib.text.Text objects
        (or possibly any matplotlib Artist will work)
        """
        self.artistlist = artistlist

    def remove(self):
        for T in self.artistlist:
            T.remove()

    def add_to_axes(self, ax):
        for T in self.artistlist:
            ax.add_artist(T)

    def get_visible(self):
        visible = True
        for T in self.artistlist:
            if not T.get_visible():
                visible = False
        return visible

    def set_visible(self, visible=True):
        for T in self.artistlist:
            T.set_visible(visible)

    def set_zorder(self, zorder):
        for T in self.artistlist:
            T.set_zorder(zorder)


def flatten_header(header):
    """
    Attempt to turn an N-dimensional fits header into a 2-dimensional header
    Turns all CRPIX[>2] etc. into new keywords with suffix 'A'
    """

    orig_wcs = wcs.WCS(header)
    newheader = orig_wcs.celestial.to_header()
    newheader['NAXIS'] = 2
    newheader['NAXIS1'] = header['NAXIS{0}'.format(orig_wcs.wcs.lng + 1)]
    newheader['NAXIS2'] = header['NAXIS{0}'.format(orig_wcs.wcs.lat + 1)]

    return newheader



from .decorators import auto_refresh


class Ticks(object):

    @auto_refresh
    def __init__(self, parent):
        self._figure = parent._figure
        self._ax = parent.ax
        self.x = parent.x
        self.y = parent.y
        self._wcs = self._ax.wcs

    def set_tick_direction(self, direction):
        """
        Set the direction of the ticks to be facing out of the axes (``out``)
        or into the axes (``in``).
        """
        if direction in ('in', 'out'):
            self._ax.coords[self.x].ticks.set_tick_out(direction == 'out')
            self._ax.coords[self.y].ticks.set_tick_out(direction == 'out')
        else:
            raise ValueError("direction should be 'in' or 'out'")

    @auto_refresh
    def set_xspacing(self, spacing):
        """
        Set the x-axis tick spacing, in degrees. To set the tick spacing to be
        automatically determined, set this to 'auto'.
        """
        self._set_spacing(self.x, spacing)

    @auto_refresh
    def set_yspacing(self, spacing):
        """
        Set the y-axis tick spacing, in degrees. To set the tick spacing to be
        automatically determined, set this to 'auto'.
        """
        self._set_spacing(self.y, spacing)

    @auto_refresh
    def _set_spacing(self, coord, spacing):
        if spacing == 'auto':
            self._ax.coords[coord].set_ticks(spacing=None)
        else:
            coord_unit = self._wcs.wcs.cunit[coord]
            self._ax.coords[coord].set_ticks(spacing=spacing * coord_unit)

    @auto_refresh
    def set_color(self, color):
        """
        Set the color of the ticks
        """
        self._ax.coords[self.x].set_ticks(color=color)
        self._ax.coords[self.y].set_ticks(color=color)

    @auto_refresh
    def set_length(self, length, minor_factor=0.5):
        """
        Set the length of the ticks (in points)
        """
        # TODO: Can't set minor ticksize. Should we just remove that?
        # Not mentioned in the APLpy in docs either
        self._ax.coords[self.x].set_ticks(size=length)
        self._ax.coords[self.y].set_ticks(size=length)

    @auto_refresh
    def set_linewidth(self, linewidth):
        """
        Set the linewidth of the ticks (in points)
        """
        self._ax.coords[self.x].set_ticks(width=linewidth)
        self._ax.coords[self.y].set_ticks(width=linewidth)

    @auto_refresh
    def set_minor_frequency(self, xfrequency, yfrequency=None):
        '''
        Set the number of subticks per major tick.

        Set to one to hide minor ticks. If not yfrequency given, frequency is
        the same in both axis. Otherwise, xfrequency represents the frequency in
        the xaxis and yfrequency the one in the yaxis.
        '''

        if yfrequency is None:
            yfrequency = xfrequency

        self._ax.coords[self.x].set_minor_frequency(xfrequency)
        self._ax.coords[self.y].set_minor_frequency(yfrequency)

    @auto_refresh
    def show(self):
        """
        Show the x- and y-axis ticks
        """
        self.show_x()
        self.show_y()

    @auto_refresh
    def hide(self):
        """
        Hide the x- and y-axis ticks
        """
        self.hide_x()
        self.hide_y()

    @auto_refresh
    def show_x(self):
        """
        Show the x-axis ticks
        """
        self._ax.coords[self.x].set_ticks_visible(True)

    @auto_refresh
    def hide_x(self):
        """
        Hide the x-axis ticks
        """
        self._ax.coords[self.x].set_ticks_visible(False)

    @auto_refresh
    def show_y(self):
        """
        Show the y-axis ticks
        """
        self._ax.coords[self.y].set_ticks_visible(True)

    @auto_refresh
    def hide_y(self):
        """
        Hide the y-axis ticks
        """
        self._ax.coords[self.y].set_ticks_visible(False)
